package de.intevation.flys.model;

import java.io.Serializable;

import java.util.List;

import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.persistence.GeneratedValue;
import javax.persistence.Column;
import javax.persistence.SequenceGenerator;
import javax.persistence.GenerationType;
import javax.persistence.OneToOne;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.JoinColumn;

import java.math.MathContext;
import java.math.BigDecimal;

import org.hibernate.Session;
import org.hibernate.Query;

import de.intevation.flys.backend.SessionHolder;

@Entity
@Table(name = "cross_sections")
public class CrossSection
implements   Serializable
{
    public static final MathContext PRECISION = new MathContext(6);

    private Integer                id;
    private River                  river;
    private TimeInterval           timeInterval;
    private String                 description;
    private List<CrossSectionLine> lines;

    public CrossSection() {
    }

    public CrossSection(
        River        river,
        TimeInterval timeInterval,
        String       description
    ) {
        this.river        = river;
        this.timeInterval = timeInterval;
        this.description  = description;
    }

    @Id
    @SequenceGenerator(
        name           = "SEQUENCE_CROSS_SECTIONS_ID_SEQ",
        sequenceName   = "CROSS_SECTIONS_ID_SEQ",
        allocationSize = 1)
    @GeneratedValue(
        strategy  = GenerationType.SEQUENCE,
        generator = "SEQUENCE_CROSS_SECTIONS_ID_SEQ")
    @Column(name = "id")
    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    @OneToOne
    @JoinColumn(name = "river_id")
    public River getRiver() {
        return river;
    }

    public void setRiver(River river) {
        this.river = river;
    }

    @OneToOne
    @JoinColumn(name = "time_interval_id")
    public TimeInterval getTimeInterval() {
        return timeInterval;
    }

    public void setTimeInterval(TimeInterval timeInterval) {
        this.timeInterval = timeInterval;
    }

    @Column(name = "description")
    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    @OneToMany
    @OrderBy("km")
    @JoinColumn(name="cross_section_id")
    public List<CrossSectionLine> getLines() {
        return lines;
    }

    public void setLines(List<CrossSectionLine> lines) {
        this.lines = lines;
    }

    public List<CrossSectionLine> getLines(double startKm, double endKm) {
        Session session = SessionHolder.HOLDER.get();
        Query query = session.createQuery(
            "from CrossSectionLine where crossSection=:crossSection " +
            "and km between :startKm and :endKm order by km");
        query.setParameter("crossSection", this);
        query.setParameter("startKm", new BigDecimal(startKm, PRECISION));
        query.setParameter("endKm", new BigDecimal(endKm, PRECISION));

        return query.list();
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
